@@ -18,9 +18,10 @@ from urllib.error import URLError
 from urllib.parse import urlsplit
 from urllib.request import urlopen
 
+import aiofiles
+import aiohttp
 import asyncpraw.exceptions
 import asyncprawcore
-import httpx
 from asyncpraw.models import Submission
 from bs4 import BeautifulSoup
 from gfycat.client import GfycatClient
@@ -30,7 +31,8 @@ from imgurpython.helpers.error import ImgurClientError
 from PIL import Image as PILImage
 from PIL import UnidentifiedImageError
 from rich import print as rprint
-from tqdm.asyncio import tqdm
+from tqdm import tqdm
+from tqdm.asyncio import tqdm as aiotqdm
 
 from . import PROGRESS_BAR_FORMAT
 from . import USER_AGENT
@@ -59,47 +61,54 @@ async def get_file(img_url: str, file_path: str, progress_label: str) -> Optiona
     Returns:
         file_path (string): path to downloaded image or None if no image was downloaded
     """
-    logger.debug("!!! 🎇🎇 Async Get File %s from %s", file_path, img_url)
+    logger.debug("!!! Async Get File %s from %s", file_path, img_url)
+    chunk_size = 64 * 1024
     try:
-        client = httpx.AsyncClient()
+        client = aiohttp.ClientSession(raise_for_status=True)
         meta = await client.head(url=img_url)
-        size = int(meta.headers["content-length"])
-        async with client.stream(method="GET", url=img_url) as response:
-            with open(file_path, "wb") as file_out:
-                async for chunk in tqdm(
-                    iterable=response.aiter_bytes(1),
-                    unit="iB",
-                    unit_scale=True,
-                    unit_divisor=1024,
-                    desc=f"{progress_label:.<60}",
-                    ncols=120,
-                    bar_format=PROGRESS_BAR_FORMAT,
-                    total=size,
-                ):
-                    file_out.write(chunk)
-            return file_path
+        download_size = int(meta.headers["content-length"])
 
-    except httpx.HTTPError as save_image_error:
+        file_out = await aiofiles.open(file=file_path, mode="wb")
+        progress_bar = tqdm(
+            unit="iB",
+            unit_scale=True,
+            unit_divisor=1024,
+            desc=f"{progress_label:.<60}",
+            ncols=120,
+            bar_format=PROGRESS_BAR_FORMAT,
+            total=download_size,
+        )
+        response = await client.get(url=img_url)
+        async for data_chunk in response.content.iter_chunked(chunk_size):
+            await file_out.write(data_chunk)
+            progress_bar.update(len(data_chunk))
+
+        await file_out.close()
+        await client.close()
+        await asyncio.sleep(0)  # allow client session to close before continuing
+        return file_path
+
+    except aiohttp.ClientError as save_image_error:
         logger.error("Failed to download (%s): %s", img_url, save_image_error)
         return None
 
 
-def get_secrets() -> Dict[str, Secret]:
+async def get_secrets(config_dir: str) -> Dict[str, Secret]:
     """Helper method to collect all api secrets either from already store
     secrets files or from user input."""
 
     secrets = {}
-    secrets["reddit"] = _get_reddit_secret()
-    secrets["gfycat"] = _get_gfycat_secret()
-    secrets["imgur"] = _get_imgur_secret()
+    secrets["reddit"] = await _get_reddit_secret(config_dir=config_dir)
+    secrets["gfycat"] = _get_gfycat_secret(config_dir=config_dir)
+    secrets["imgur"] = _get_imgur_secret(config_dir=config_dir)
 
     return secrets
 
 
-def _get_reddit_secret() -> Secret:
+async def _get_reddit_secret(config_dir: str) -> Secret:
     """Helper method to collect reddit secret from stored secrets file or from
     user input."""
-    reddit_secrets_file = "reddit.secret"
+    reddit_secrets_file = f"{config_dir}/reddit.secret"  # pragma: allowlist secret
     reddit_config = configparser.ConfigParser()
     if not os.path.exists(reddit_secrets_file):
         rprint("Reddit API keys not found. Please provide Reddit API values.")
@@ -118,8 +127,8 @@ def _get_reddit_secret() -> Secret:
                 client_id=reddit_agent,
                 client_secret=reddit_client_secret,
             )
-            subreddit = reddit_client.subreddit("announcements")
-            for _post in subreddit.hot():
+            subreddit = await reddit_client.subreddit("announcements")
+            async for _post in subreddit.hot():
                 continue
 
             # It worked, so save the keys to a file
@@ -143,7 +152,7 @@ def _get_reddit_secret() -> Secret:
     )
 
 
-def _get_gfycat_secret() -> Secret:
+def _get_gfycat_secret(config_dir: str) -> Secret:
     """_get_gfycat_secrets checks if the Gfycat api secrets file exists.
 
     - If the file exists, this method reads the files and returns the secrets
@@ -151,15 +160,12 @@ def _get_gfycat_secret() -> Secret:
     - If the file doesn't exist it asks the user over stdin to supply these values
       and then saves them into the gfycat_secrets file
 
-    Arguments:
-        logger (logging.Logger): logger to use for general logging messages
-
     Returns:
         api_secrets (Secret): instance of Secret class containing the api secrets
         to work with gyfycat
     """
 
-    secrets_file = "gfycat.secret"
+    secrets_file = f"{config_dir}/gfycat.secret"  # pragma: allowlist secret
     if not os.path.exists(secrets_file):
         rprint("Gfycat API keys not found. (See wiki if you need help).")
 
@@ -197,7 +203,7 @@ def _get_gfycat_secret() -> Secret:
     )
 
 
-def _get_imgur_secret() -> Secret:
+def _get_imgur_secret(config_dir: str) -> Secret:
     """_get_imgur_secrets checks if the Imgur api secrets file exists.
 
     - If the file exists, this method reads the imgur secrets file and returns the
@@ -205,14 +211,11 @@ def _get_imgur_secret() -> Secret:
     - If the file doesn't exist it asks the user over stdin to supply these values
       and then saves them into the imgur_secrets file
 
-    Arguments:
-        logger (logging.Logger): logger to use for general logging messages
-
     Returns:
         api_secrets (Secret): instance of Secret class containing the api secrets
         to work with imgut
     """
-    secrets_file = "imgur.secret"
+    secrets_file = f"{config_dir}/imgur.secret"  # pragma: allowlist secret
     if not os.path.exists(secrets_file):
         rprint("Imgur API keys not found. (See wiki if you need help).")
 
@@ -274,7 +277,7 @@ class RedditHelper:
         for subreddit in self.config.subreddits:
             tasks.append(self.get_reddit_posts(subreddit, reddit_con))
         progress_title = "Processing Subreddits"
-        task_results = await tqdm.gather(
+        task_results = await aiotqdm.gather(
             *tasks,
             desc=f"{progress_title:.<60}",
             ncols=120,
@@ -311,7 +314,7 @@ class RedditHelper:
             )
         return posts
 
-    def winnow_reddit_posts(self: RH) -> None:
+    async def winnow_reddit_posts(self: RH) -> None:
         """Filters out reddit posts according to configuration and whether it
         has already been tooted."""
         recorder = self.config.bot.post_recorder
@@ -355,12 +358,12 @@ class RedditHelper:
                     del posts[post_id]
                     continue
 
-                if recorder.duplicate_check(post_id):
+                if await recorder.duplicate_check(post_id):
                     logger.debug("Skipping %s, it has already been tooted", post_id)
                     del posts[post_id]
                     continue
 
-                if recorder.duplicate_check(posts[post_id].url):
+                if await recorder.duplicate_check(posts[post_id].url):
                     logger.debug("Skipping %s, it has already been tooted", post_id)
                     del posts[post_id]
                     continue
@@ -582,17 +585,19 @@ class LinkedMediaHelper:
         file_path = self.save_dir + "/"
         try:
             gfycat_name = os.path.basename(urlsplit(img_url).path)
-            response = httpx.get(img_url)
-            response.raise_for_status()
-            soup = BeautifulSoup(response.text, "lxml")
+            async with aiohttp.ClientSession(raise_for_status=True) as session:
+                async with session.get(url=img_url) as response:
+                    soup = BeautifulSoup(response.text, "lxml")
+
             for tag in soup.find_all("source", src=True):
                 src = tag["src"]
                 if "giant" in src and "mp4" in src:
                     gfycat_url = src
             file_path += gfycat_name + ".mp4"
         except (
-            httpx.HTTPError,
+            aiohttp.ClientError,
             GfycatClientError,
+            TypeError,
         ) as gfycat_error:
             logger.error("Error downloading Gfycat link: %s", gfycat_error)
             return None
@@ -824,10 +829,10 @@ class MediaAttachment:
             logger.debug("Media path for checksum calculation: %s", media_path)
             if media_path:
                 sha256 = hashlib.sha256()
-                with open(media_path, "rb") as media_file:
+                async with aiofiles.open(file=media_path, mode="rb") as media_file:
                     # Read and update hash string value in blocks of 64K
                     while True:
-                        data = media_file.read(2**16)
+                        data = await media_file.read(2**16)
                         if not data:
                             break
                         sha256.update(data)
