@@ -2,20 +2,24 @@
 if a reddit post should be published on Mastodon."""
 import configparser
 import logging
-import sqlite3
 import sys
 from dataclasses import dataclass
 from typing import Any
 from typing import List
 from typing import Optional
+from typing import Type
 from typing import TypeVar
 
+import aiosqlite
 from rich.logging import RichHandler
 
+from . import __display_name__
 from . import POST_RECORDER_SQLITE_DB
 
 PR = TypeVar("PR", bound="PostRecorder")
-Config = TypeVar("Config", bound="Configuration")
+ConfigClass = TypeVar("ConfigClass", bound="Configuration")
+
+logger = logging.getLogger(__display_name__)
 
 
 class PostRecorder:
@@ -23,29 +27,30 @@ class PostRecorder:
     checking against the log of published content to determine if a post would
     be a duplicate."""
 
-    def __init__(self: PR, cache_file: str, logger: logging.Logger) -> None:
-        self.cache_file = cache_file
-        self.logger = logger
-        self.db_connection = sqlite3.connect(POST_RECORDER_SQLITE_DB)
+    def __init__(self: PR, history_db_dir: str = ".") -> None:
+        self.history_db_file = f"{history_db_dir.rstrip('/')}/{POST_RECORDER_SQLITE_DB}"
+        self.history_db: Optional[aiosqlite.Connection] = None
 
+    async def db_init(self: PR) -> None:
+        """Initialises DB connection and tables if necessary."""
+        self.history_db = await aiosqlite.connect(database=self.history_db_file)
         # Make sure DB tables exist
-        db_cursor = self.db_connection.cursor()
-        db_cursor.execute(
+        await self.history_db.execute(
             "CREATE TABLE IF NOT EXISTS post (id TEXT PRIMARY KEY) WITHOUT ROWID"
         )
-        db_cursor.execute(
+        await self.history_db.execute(
             "CREATE TABLE IF NOT EXISTS share "
             "(url TEXT PRIMARY KEY) "
             "WITHOUT ROWID"
         )
-        db_cursor.execute(
+        await self.history_db.execute(
             "CREATE TABLE IF NOT EXISTS hash "
             "(checksum TEXT PRIMARY KEY) "
             "WITHOUT ROWID"
         )
-        self.db_connection.commit()
+        await self.history_db.commit()
 
-    def duplicate_check(self: PR, identifier: str) -> bool:
+    async def duplicate_check(self: PR, identifier: str) -> bool:
         """Checks if identifier can be found in log file of content posted to
         Mastodon.
 
@@ -61,32 +66,34 @@ class PostRecorder:
                 Mastodon
                 True if "identifier" has been found in log of content.
         """
-        db_cursor = self.db_connection.cursor()
+
+        if self.history_db is None:
+            return False
 
         # check for reddit_id
-        result = db_cursor.execute(
+        cursor = await self.history_db.execute(
             "SELECT * FROM post where id=:id", {"id": identifier}
         )
-        if result.fetchone():
+        if await cursor.fetchone():
             return True
 
         # check for Shared URL
-        result = db_cursor.execute(
+        cursor = await self.history_db.execute(
             "SELECT * FROM share where url=:url", {"url": identifier}
         )
-        if result.fetchone():
+        if await cursor.fetchone():
             return True
 
         # check for attachment hash
-        result = db_cursor.execute(
+        cursor = await self.history_db.execute(
             "SELECT * FROM hash where checksum=:checksum", {"checksum": identifier}
         )
-        if result.fetchone():
+        if await cursor.fetchone():
             return True
 
         return False
 
-    def log_post(
+    async def log_post(
         self: PR,
         reddit_id: Optional[str] = None,
         shared_url: Optional[str] = None,
@@ -103,14 +110,21 @@ class PostRecorder:
                 Checksum of media attachment that was shared on Mastodon.
                 This enables checking for duplicate media even if file has been renamed.
         """
-        db_cursor = self.db_connection.cursor()
+        if self.history_db is None:
+            return
+
         if reddit_id:
-            db_cursor.execute("INSERT INTO post VALUES (?)", (reddit_id,))
+            await self.history_db.execute("INSERT INTO post VALUES (?)", (reddit_id,))
         if shared_url:
-            db_cursor.execute("INSERT INTO share VALUES (?)", (shared_url,))
+            await self.history_db.execute("INSERT INTO share VALUES (?)", (shared_url,))
         if check_sum:
-            db_cursor.execute("INSERT INTO hash VALUES (?)", (check_sum,))
-        self.db_connection.commit()
+            await self.history_db.execute("INSERT INTO hash VALUES (?)", (check_sum,))
+        await self.history_db.commit()
+
+    async def close_db(self: PR) -> None:
+        """Closes db connection."""
+        if self.history_db:
+            await self.history_db.close()
 
 
 @dataclass
@@ -126,13 +140,11 @@ class BotConfig:
     """Dataclass holding configuration values for general behaviour of
     tootbot."""
 
-    cache_file: str
     post_recorder: PostRecorder
     delay_between_posts: int
     run_once_only: bool
     hash_tags: List[str]
     log_level: str
-    logger: logging.Logger
 
 
 @dataclass
@@ -184,6 +196,7 @@ class MastodonConfig:
 
     domain: str
     media_always_sensitive: bool
+    post_visibility: str
     throttling_enabled: bool
     throttling_max_delay: int
     number_of_errors: int
@@ -210,95 +223,134 @@ class Configuration:
     mastodon_config: MastodonConfig
     reddit: RedditReaderConfig
 
-    def __init__(self: Config) -> None:
+    @classmethod
+    async def load_config(
+        cls: Type[ConfigClass],
+        config_dir: str,
+        debug_log: str,
+    ) -> ConfigClass:
+        """Loads aor queries config parameters from user, as necessary."""
+
+        if debug_log:
+            file_log_formatter = logging.Formatter(
+                "%(asctime)s %(name)s[%(process)d] %(levelname)s %(message)s",
+                datefmt="%H:%M:%S",
+            )
+            file_handler = logging.FileHandler(filename=debug_log)
+            file_handler.setFormatter(file_log_formatter)
+            file_handler.setLevel(logging.DEBUG)
+            logger.addHandler(file_handler)
 
         # Make sure config file exists
         try:
             config = configparser.ConfigParser()
-            config.read("config.ini")
+            config.read(f"{config_dir}/config.ini")
         except configparser.Error as config_error:
             print("[ERROR] Error while reading config file: %s", config_error)
             sys.exit(1)
 
-        # Set-up logging
-        log_level = "INFO"
-        if config["BotSettings"]["LogLevel"]:
-            log_level = config["BotSettings"]["LogLevel"]
-        logging.basicConfig(
-            level=log_level,
-            format="%(name)s[%(process)d] %(levelname)s %(message)s",
-            datefmt="%H:%M:%S",
-            handlers=[RichHandler()],
+        await cls._set_up_logging(config)
+        bot = await Configuration._load_bot_settings(
+            config=config,
+            config_dir=config_dir,
         )
-        logger = logging.getLogger("Tootbot")
+        reddit = RedditReaderConfig(
+            post_limit=int(config["BotSettings"]["PostLimit"]),
+            nsfw_allowed=str_to_bool(config["BotSettings"]["NSFWPostsAllowed"]),
+            nsfw_marked=str_to_bool(config["BotSettings"]["NSFWPostsMarked"]),
+            spoilers=str_to_bool(config["BotSettings"]["SpoilersAllowed"]),
+            self_posts=str_to_bool(config["BotSettings"]["SelfPostsAllowed"]),
+            stickied_allowed=str_to_bool(config["BotSettings"]["StickiedPostsAllowed"]),
+        )
+        promo = PromoConfig(
+            every=int(config["PromoSettings"]["PromoEvery"]),
+            message=config["PromoSettings"]["PromoMessage"],
+        )
+        health = await Configuration._load_healthchecks_settings(config)
+        media = MediaConfig(
+            folder=config["MediaSettings"]["MediaFolder"],
+            media_only=str_to_bool(config["MediaSettings"]["MediaPostsOnly"]),
+        )
+        mastodon_config = MastodonConfig(
+            domain=config["Mastodon"]["InstanceDomain"],
+            media_always_sensitive=str_to_bool(config["Mastodon"]["SensitiveMedia"]),
+            post_visibility=config["Mastodon"].get("PostVisibility", "public"),
+            throttling_enabled=str_to_bool(config["Mastodon"]["ThrottlingEnabled"]),
+            throttling_max_delay=int(config["Mastodon"]["ThrottlingMaxDelay"]),
+            number_of_errors=0,
+        )
+        subreddits = await Configuration._load_subreddits_settings(config)
 
-        # Bot settings
-        bot_settings = config["BotSettings"]
+        configuration = cls(
+            bot=bot,
+            subreddits=subreddits,
+            promo=promo,
+            health=health,
+            media=media,
+            mastodon_config=mastodon_config,
+            reddit=reddit,
+        )
+        logger.debug("After loading of config: %s", configuration)
+        return configuration
+
+    @staticmethod
+    async def _load_subreddits_settings(
+        config: configparser.ConfigParser,
+    ) -> List[SubredditConfig]:
+        subreddits = []
+        for subreddit, hashtags in config.items("Subreddits"):
+            subreddits.append(SubredditConfig(subreddit, hashtags))
+        return subreddits
+
+    @staticmethod
+    async def _load_healthchecks_settings(
+        config: configparser.ConfigParser,
+    ) -> HealthCheckConfig:
+        health_enabled = False
+        if len(config["HealthChecks"]["BaseUrl"]) > 0:
+            health_enabled = True
+        health = HealthCheckConfig(
+            enabled=health_enabled,
+            base_url=config["HealthChecks"]["BaseUrl"],
+            uuid=config["HealthChecks"]["UUID"],
+        )
+        return health
+
+    @staticmethod
+    async def _load_bot_settings(
+        config: configparser.ConfigParser,
+        config_dir: str,
+    ) -> BotConfig:
         hash_tags = []
         if config["BotSettings"]["Hashtags"]:
             # Parse list of hashtags
             hash_tags_string = config["BotSettings"]["Hashtags"]
             hash_tags = [x.strip() for x in hash_tags_string.split(",")]
-        self.bot = BotConfig(
-            cache_file=bot_settings["CacheFile"],
-            post_recorder=PostRecorder(bot_settings["CacheFile"], logger),
-            delay_between_posts=int(bot_settings["DelayBetweenPosts"]),
-            run_once_only=str_to_bool(bot_settings["RunOnceOnly"]),
+        bot = BotConfig(
+            post_recorder=PostRecorder(history_db_dir=config_dir),
+            delay_between_posts=int(config["BotSettings"]["DelayBetweenPosts"]),
+            run_once_only=str_to_bool(config["BotSettings"]["RunOnceOnly"]),
             hash_tags=hash_tags,
-            log_level=bot_settings["LogLevel"],
-            logger=logger,
+            log_level=config["BotSettings"]["LogLevel"],
         )
+        await bot.post_recorder.db_init()
+        return bot
 
-        # Settings related to reddit reader
-        self.reddit = RedditReaderConfig(
-            post_limit=int(bot_settings["PostLimit"]),
-            nsfw_allowed=str_to_bool(bot_settings["NSFWPostsAllowed"]),
-            nsfw_marked=str_to_bool(bot_settings["NSFWPostsMarked"]),
-            spoilers=str_to_bool(bot_settings["SpoilersAllowed"]),
-            self_posts=str_to_bool(bot_settings["SelfPostsAllowed"]),
-            stickied_allowed=str_to_bool(bot_settings["StickiedPostsAllowed"]),
+    @staticmethod
+    async def _set_up_logging(config: configparser.ConfigParser) -> None:
+        # Set-up logging
+        log_level = "INFO"
+        if config["BotSettings"]["LogLevel"]:
+            log_level = config["BotSettings"]["LogLevel"]
+
+        std_out_formatter = logging.Formatter(
+            "%(name)s[%(process)d] %(levelname)s %(message)s",
+            datefmt="%H:%M:%S",
         )
-
-        # Settings related to promotional messages
-        promo_settings = config["PromoSettings"]
-        self.promo = PromoConfig(
-            every=int(promo_settings["PromoEvery"]),
-            message=promo_settings["PromoMessage"],
-        )
-
-        # HealthChecks related settings
-        healthchecks_settings = config["HealthChecks"]
-        health_enabled = False
-        if len(healthchecks_settings["BaseUrl"]) > 0:
-            health_enabled = True
-        self.health = HealthCheckConfig(
-            enabled=health_enabled,
-            base_url=healthchecks_settings["BaseUrl"],
-            uuid=healthchecks_settings["UUID"],
-        )
-
-        # Settings related to media attachments
-        media_settings = config["MediaSettings"]
-        self.media = MediaConfig(
-            folder=media_settings["MediaFolder"],
-            media_only=str_to_bool(media_settings["MediaPostsOnly"]),
-        )
-
-        # Mastodon info
-        mastodon_settings = config["Mastodon"]
-        self.mastodon_config = MastodonConfig(
-            domain=mastodon_settings["InstanceDomain"],
-            media_always_sensitive=str_to_bool(mastodon_settings["SensitiveMedia"]),
-            throttling_enabled=str_to_bool(mastodon_settings["ThrottlingEnabled"]),
-            throttling_max_delay=int(mastodon_settings["ThrottlingMaxDelay"]),
-            number_of_errors=0,
-        )
-
-        self.subreddits = []
-        for subreddit, hashtags in config.items("Subreddits"):
-            self.subreddits.append(SubredditConfig(subreddit, hashtags))
-
-        logger.debug("After loading of config: %s", self)
+        std_out_handler = RichHandler()
+        std_out_handler.setFormatter(std_out_formatter)
+        std_out_handler.setLevel(log_level)
+        logger.addHandler(std_out_handler)
 
 
 def str_to_bool(value: Any) -> bool:
